///////////////////////////////////////////////////////////////////////////////
// File: TestExe.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2005 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All 
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors 
// shall not be liable for any damages suffered by licensee as a result of 
// using the software. In no event will JetByte Limited be liable for any 
// lost revenue, profit or data, or for direct, indirect, special, 
// consequential, incidental or punitive damages, however caused and regardless 
// of the theory of liability, arising out of the use of or inability to use 
// software, even if JetByte Limited has been advised of the possibility of 
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include <windows.h>

#if (_MSC_VER < 1400)			// Need this for anything earlier than VS2005
#define _STLP_NEW_PLATFORM_SDK
#endif

#define _REENTRANT 1

#include <iostream>

using std::string;
using std::cout;
using std::endl;

///////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////

static string GetModuleFileName(
   HINSTANCE hModule = 0)
{
   string name = "UNAVAILABLE";

   char moduleFileName[MAX_PATH + 1] ;
	DWORD moduleFileNameLen = MAX_PATH ;

   if (::GetModuleFileName(hModule, moduleFileName, moduleFileNameLen))
   {
      name = moduleFileName;
   }

   return name;
}

static string GetModulePathName(
   HINSTANCE hModule = 0)
{
   string path = GetModuleFileName(hModule);

   string::size_type pos = path.find_last_of("\\/:");

   return path.substr(0, pos);
}

static void Tick()
{
   for (size_t i = 0; i < 10; ++i)
   {
      cout << GetTickCount() << endl;

      ::Sleep(1000);
   }
}

///////////////////////////////////////////////////////////////////////////////
// Program Entry Point
///////////////////////////////////////////////////////////////////////////////

int main(int /*argc*/, char * /*argv[ ]*/)
{
   Tick();


   STARTUPINFO startupInfo;

   ZeroMemory(&startupInfo, sizeof(startupInfo));

   startupInfo.cb = sizeof(startupInfo);

   PROCESS_INFORMATION processInfo;
   
   ZeroMemory(&processInfo, sizeof(processInfo));

   const string processName = GetModulePathName() + "\\ParentTestExe.exe";

   const string commandLine = "\"" + processName + "\"";

   cout << "About to launch: \"" << processName.c_str() << "\"" << endl;

   if (0 == ::CreateProcess(
      processName.c_str(),
      const_cast<TCHAR*>(commandLine.c_str()),
      0,                   // process attributes
      0,                   // thread attributes
      FALSE,               // inherit handles
      CREATE_NEW_CONSOLE,  // creation flags 
      0,                   // environment
      0,
      &startupInfo,
      &processInfo))
   {
      const DWORD lastError = ::GetLastError();

      cout << "Failed to launch child: " << lastError << endl;
   }
   else
   {
      cout << "Launched child" << endl;
   }

   Tick();

   ::Sleep(5000);

   return 0;
}

///////////////////////////////////////////////////////////////////////////////
// End of file: TestExe.cpp
///////////////////////////////////////////////////////////////////////////////

